﻿using ProjectCleaningCar.Entities;

namespace ProjectCleaningCar.Drawning;
/// <summary>
/// Класс, отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
public class DrawningCleaningCar : DrawningTruck
{
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="tank">Бак с водой</param>
    /// <param name="sweepingBrush">Подметательная щётка</param>
    /// <param name="flashlight">Проблескового маячок</param>
    public DrawningCleaningCar(int speed, double weight, Color bodyColor, Color additionalColor, bool tank, bool sweepingBrush, bool flashlight) : base(132, 65)
    {
        EntityTruck = new EntityCleaningCar(speed, weight, bodyColor, additionalColor, tank, sweepingBrush, flashlight);
    }

    /// <summary>
    /// Конструктор для Extention
    /// </summary>
    /// <param name="cleaningCar"></param>
    public DrawningCleaningCar(EntityCleaningCar cleaningCar) : base(132, 65)
    {
        if (cleaningCar == null) return;
        EntityTruck = new EntityCleaningCar(cleaningCar.Speed,cleaningCar.Weight, cleaningCar.BodyColor,
            cleaningCar.AdditionalColor, cleaningCar.Tank, cleaningCar.SweepingBrush, cleaningCar.Flashlight);
    }
    /// <summary>
    /// Отрисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public override void DrawTransport(Graphics g)
    {
        if (EntityTruck == null || EntityTruck is not EntityCleaningCar cleaningCar || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }
        base.DrawTransport(g);
        Pen pen = new(Color.Black);
        Brush bodyBrush = new SolidBrush(EntityTruck.BodyColor);
        Brush additionalBrush = new SolidBrush(cleaningCar.AdditionalColor);
        //бак
        if (cleaningCar.Tank)
        {
            g.FillRectangle(additionalBrush, _startPosX.Value, _startPosY.Value, 98, 38);
            g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value, 98, 38);
            g.FillRectangle(bodyBrush, _startPosX.Value, _startPosY.Value + 10, 98, 10);
            g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 10, 98, 10);
        }
        //подметательная щётка
        if (cleaningCar.SweepingBrush)
        {
            Pen additionalPen = new(cleaningCar.AdditionalColor);
            g.DrawEllipse(additionalPen, _startPosX.Value + 122, _startPosY.Value + 45, 18, 18);
            g.FillEllipse(additionalBrush, _startPosX.Value + 122, _startPosY.Value + 45, 18, 18);
            g.FillEllipse(bodyBrush, _startPosX.Value + 126, _startPosY.Value + 49, 10, 10);
            g.DrawLine(pen, _startPosX.Value + 117, _startPosY.Value + 45, _startPosX.Value + 131, _startPosY.Value + 54);
        }
        //проблесковый маячок
        if (cleaningCar.Flashlight)
        {
            g.FillRectangle(additionalBrush, _startPosX.Value + 105, _startPosY.Value, 4, 7);
        }
        //диски
        g.FillEllipse(additionalBrush, _startPosX.Value + 15, _startPosY.Value + 50, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 15, _startPosY.Value + 50, 10, 10);

        g.FillEllipse(additionalBrush, _startPosX.Value + 40, _startPosY.Value + 50, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 40, _startPosY.Value + 50, 10, 10);

        g.FillEllipse(additionalBrush, _startPosX.Value + 102, _startPosY.Value + 50, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 102, _startPosY.Value + 50, 10, 10);
        //окно
        g.DrawRectangle(pen, _startPosX.Value + 108, _startPosY.Value + 11, 10, 11);
        g.FillRectangle(additionalBrush, _startPosX.Value + 108, _startPosY.Value + 11, 10, 11);
    }
}
