﻿namespace ProjectCleaningCar.Entities;
/// <summary>
/// Класс-сущность "Подметательно-уборочная машина"
/// </summary>
public class EntityCleaningCar : EntityTruck
{
    /// <summary>
    /// Дополнительный цвет (для опциональных элементов)
    /// </summary>
    public Color AdditionalColor { get; private set; }
    public void SetAdditionalColor(Color additionalColor)
    {
        AdditionalColor = additionalColor;
    }
    /// <summary>
    /// Признак (опция) наличия бака
    /// </summary>
    public bool Tank { get; private set; }
    /// <summary>
    /// Признак (опция) наличия щётки
    /// </summary>
    public bool SweepingBrush { get; private set; }
    /// <summary>
    /// Признак (опция) наличия проблескового маячка
    /// </summary>
    public bool Flashlight { get; private set; }
    /// <summary>
    /// Инициализация полей объекта-класса подметально-уборочной машины
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес автомобиля</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="tank">Признак наличия бака под воду</param>
    /// <param name="sweepingBrush">Признак наличия подметательной щётки</param>
    /// <param name="flashlight">Признак наличия проблескового маячка</param>
    public EntityCleaningCar(int speed, double weight, Color bodyColor, Color additionalColor, bool tank, bool sweepingBrush, bool flashlight) : base(speed, weight, bodyColor)
    {
        AdditionalColor = additionalColor;
        Tank = tank;
        SweepingBrush = sweepingBrush;
        Flashlight = flashlight;
    }

    /// <summary>
    /// Получение строк со значениями свойств продвинутого объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public override string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityCleaningCar), Speed.ToString(), Weight.ToString(), BodyColor.Name, AdditionalColor.Name,
                        Tank.ToString(), SweepingBrush.ToString(), Flashlight.ToString()};
    }
    /// <summary>
    /// Создание продвинутого объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityCleaningCar? CreateEntityCleaningCar(string[] strs)
    {
        if (strs.Length != 8 || strs[0] != nameof(EntityCleaningCar))
        {
            return null;
        }
        return new EntityCleaningCar(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]),
                                    Color.FromName(strs[4]), Convert.ToBoolean(strs[5]), Convert.ToBoolean(strs[6]), Convert.ToBoolean(strs[7]));
    }
}
