﻿namespace ProjectCleaningCar.Entities;
/// <summary>
/// Класс-сущность "Грузовик"
/// </summary>
public class EntityTruck
{
    /// <summary>
    /// Скорость
    /// </summary>
    public int Speed { get; private set; }
    /// <summary>
    /// Вес
    /// </summary>
    public double Weight { get; private set; }
    /// <summary>
    /// Основной цвет
    /// </summary>
    public Color BodyColor { get; private set; }
    public void SetBodyColor(Color bodyColor)
    {
        BodyColor = bodyColor;
    }
    /// <summary>
    /// Шаг перемещения грузовика
    /// </summary>
    public double Step => Speed * 200 / Weight;

    /// <summary>
    /// Инициализация полей объекта-класса грузовик
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес автомобиля</param>
    /// <param name="bodyColor">Основной цвет</param>
    public EntityTruck(int speed, double weight, Color bodyColor)
    {
        Speed = speed;
        Weight = weight;
        BodyColor = bodyColor;
    }

    /// <summary>
    /// Получение строк со значениями свойств объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public virtual string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityTruck), Speed.ToString(), Weight.ToString(), BodyColor.Name };
    }

    /// <summary>
    /// Создание объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityTruck? CreateEntityCar(string[] strs)
    {
        if (strs.Length != 4 || strs[0] != nameof(EntityTruck))
        {
            return null;
        }
        return new EntityTruck(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]));
    }
}