﻿using ProjectCleaningCar.Drawning;
using ProjectCleaningCar.Entities;

namespace ProjectCleaningCar;

/// <summary>
/// Форма конфигурации объекта
/// </summary>
public partial class FormTruckConfig : Form
{
    /// <summary>
    /// Объект прорисовки грузовика
    /// </summary>
    private DrawningTruck? _truck;

    /// <summary>
    /// Событие для передачи объекта
    /// </summary>
    public event Action<DrawningTruck>? _truckDelegate;

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormTruckConfig()
    {
        InitializeComponent();

        panelRed.MouseDown += Panel_MouseDown;
        panelGreen.MouseDown += Panel_MouseDown;
        panelBlue.MouseDown += Panel_MouseDown;
        panelYellow.MouseDown += Panel_MouseDown;
        panelWhite.MouseDown += Panel_MouseDown;
        panelGray.MouseDown += Panel_MouseDown;
        panelBlack.MouseDown += Panel_MouseDown;
        panelPurple.MouseDown += Panel_MouseDown;

        buttonCancel.Click += (sender, e) => Close();

    }

    /// <summary>
    /// Привязка внешнего метода к событию
    /// </summary>
    /// <param name="truckDelegate"></param>
    public void AddEvent(Action<DrawningTruck>? truckDelegate)
    {
        _truckDelegate += truckDelegate;
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    private void DrawObject()
    {
        Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _truck?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
        _truck?.SetPosition(35, 35);
        _truck?.DrawTransport(gr);
        pictureBoxObject.Image = bmp;
    }

    /// <summary>
    /// Передаем информацию при нажатии на Label
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelObject_MouseDown(object sender, MouseEventArgs e)
    {
        (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
    }

    /// <summary>
    /// Проверка получаемой информации (ее типа на соответствие требуемому)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragEnter(object sender, DragEventArgs e)
    {
        e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
    }

    /// <summary>
    /// Действия при приеме перетаскиваемой информации
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragDrop(object sender, DragEventArgs e)
    {
        switch (e.Data?.GetData(DataFormats.Text)?.ToString())
        {
            case "labelSimpleObject":
                _truck = new DrawningTruck((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White);
                break;
            case "labelModifiedObject":
                _truck = new DrawningCleaningCar((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White,
                    Color.Black, checkBoxTank.Checked, checkBoxSweepingBrush.Checked, checkBoxFlashlight.Checked);
                break;
        }

        DrawObject();
    }

    /// <summary>
    /// Передаём информацию при нажатии на Panel
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void Panel_MouseDown(object? sender, MouseEventArgs e)
    {
        (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor ?? Color.White, DragDropEffects.Move | DragDropEffects.Copy);
    }

    /// <summary>
    /// Передача объекта в другую форму
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAdd_Click(object sender, EventArgs e)
    {
        if (_truck != null)
        {
            _truckDelegate?.Invoke(_truck);
            Close();
        }
    }

    /// <summary>
    /// Передача основного цвета
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void labelBodyColor_DragEnter(object sender, DragEventArgs e)
    {
        if (e.Data.GetDataPresent(typeof(Color)))
        {
            e.Effect = DragDropEffects.Copy;
        }
        else
        {
            e.Effect = DragDropEffects.None;
        }
    }

    /// <summary>
    /// Прорисовка основным цветом
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void labelBodyColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_truck != null)
        {
            _truck.EntityTruck?.SetBodyColor((Color)e.Data.GetData(typeof(Color)));
            DrawObject();
        }
    }

    /// <summary>
    /// Прорисовка дополнительным цветом
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void labelAdditionalColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_truck?.EntityTruck is EntityCleaningCar _cleaningCar)
        {
            _cleaningCar.SetAdditionalColor((Color)e.Data.GetData(typeof(Color)));
        }
        DrawObject();
    }

    /// <summary>
    /// Передача дополнительного цвета
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void labelAdditionalColor_DragEnter(object sender, DragEventArgs e)
    {
        if (_truck is DrawningCleaningCar)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
    }
}
