using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Serilog;


namespace ProjectCleaningCar
{
    internal static class Program
    {
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            ServiceCollection services = new();
            ConfigureService(services);
            using ServiceProvider serviceProvider = services.BuildServiceProvider();
            Application.Run(serviceProvider.GetRequiredService<FormCleaningCarCollection>());
        }

        /// <summary>
        ///  
        /// </summary>
        /// <param name="services"></param>
        private static void ConfigureService(ServiceCollection services)
        {
            services
                .AddSingleton<FormCleaningCarCollection>()
                .AddLogging(option =>
                {
                    option.SetMinimumLevel(LogLevel.Information);
                    var config = new ConfigurationBuilder()
                        .AddJsonFile("serilogConfig.json", optional: false, reloadOnChange: true)
                        .Build();
                    option.AddSerilog(Log.Logger = new LoggerConfiguration()
                        .ReadFrom.Configuration(config)
                        .CreateLogger());
                });
           
            
        }
    }
}